@@ -48,6 +48,8 @@ class Options:
         # predefined configurable sources
         FLICKR = "flickr"
 
+        WALLHAVEN = "wallhaven"
+
         BUILTIN_SOURCE_TYPES = {
             IMAGE,
             FOLDER,
@@ -99,11 +101,26 @@ class Options:
             except Exception:
                 pass
 
+            try:
+                self.internet_enabled = config["internet_enabled"].lower() in TRUTH_VALUES
+            except Exception:
+                pass
+
             try:
                 self.safe_mode = config["safe_mode"].lower() in TRUTH_VALUES
             except Exception:
                 pass
 
+            try:
+                self.set_wallpaper_script = os.path.expanduser(config["set_wallpaper_script"])
+            except Exception:
+                pass
+
+            try:
+                self.get_wallpaper_script = os.path.expanduser(config["get_wallpaper_script"])
+            except Exception:
+                pass
+
             try:
                 self.download_folder = os.path.expanduser(config["download_folder"])
             except Exception:
@@ -126,6 +143,11 @@ class Options:
             except Exception:
                 pass
 
+            try:
+                self.wallhaven_api_key = str(config["wallhaven_api_key"]).strip()
+            except Exception:
+                pass
+
             try:
                 self.favorites_folder = os.path.expanduser(config["favorites_folder"])
             except Exception:
@@ -479,7 +501,7 @@ class Options:
             for downloader in sorted(self.SIMPLE_DOWNLOADERS, key=lambda dl: dl.get_source_type()):
                 if downloader.get_source_type() not in source_types:
                     self.sources.append(
-                        [True, downloader.get_source_type(), downloader.get_description()]
+                        [False, downloader.get_source_type(), downloader.get_description()]
                     )
 
             self.parse_autosources()
@@ -587,12 +609,17 @@ class Options:
         self.change_enabled = True
         self.change_on_start = False
         self.change_interval = 300
+        self.internet_enabled = True
         self.safe_mode = False
 
+        self.set_wallpaper_script = os.path.join(get_profile_path(), "scripts", "set_wallpaper")
+        self.get_wallpaper_script = os.path.join(get_profile_path(), "scripts", "get_wallpaper")
+
         self.download_folder = os.path.join(get_profile_path(), "Downloaded")
         self.download_preference_ratio = 0.9
         self.quota_enabled = True
         self.quota_size = 1000
+        self.wallhaven_api_key = ""
 
         self.favorites_folder = os.path.join(get_profile_path(), "Favorites")
         self.favorites_operations = [
@@ -698,14 +725,20 @@ class Options:
             config["change_enabled"] = str(self.change_enabled)
             config["change_on_start"] = str(self.change_on_start)
             config["change_interval"] = str(self.change_interval)
+            config["internet_enabled"] = str(self.internet_enabled)
             config["safe_mode"] = str(self.safe_mode)
 
+            config["set_wallpaper_script"] = Util.collapseuser(self.set_wallpaper_script)
+            config["get_wallpaper_script"] = Util.collapseuser(self.get_wallpaper_script)
+
             config["download_folder"] = Util.collapseuser(self.download_folder)
             config["download_preference_ratio"] = str(self.download_preference_ratio)
 
             config["quota_enabled"] = str(self.quota_enabled)
             config["quota_size"] = str(self.quota_size)
 
+            config["wallhaven_api_key"] = str(self.wallhaven_api_key)
+
             config["favorites_folder"] = Util.collapseuser(self.favorites_folder)
             config["favorites_operations"] = ";".join(
                 ":".join(x) for x in self.favorites_operations
