@@ -13,68 +13,73 @@
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
-import bs4
-import requests
-from functools import wraps
+import base64
+import codecs
+import datetime
+import functools
+import gettext
 import hashlib
 import json
-import re
+import logging
 import os
 import random
-import logging
+import re
+import shutil
 import string
+import subprocess
+import sys
 import threading
 import time
 import urllib.parse
-import datetime
-import sys
-import functools
-import subprocess
-import codecs
+from itertools import cycle
 
+import bs4
+import requests
 from PIL import Image
 
-import gi
-gi.require_version('GExiv2', '0.10')
-gi.require_version('PangoCairo', '1.0')
+from variety_lib import get_version
 
-from gi.repository import Gdk, Pango, GdkPixbuf, GLib, GExiv2, Gio
+# fmt: off
+import gi  # isort:skip
+gi.require_version("GExiv2", "0.10")
+gi.require_version("PangoCairo", "1.0")
+gi.require_version('Gdk', '3.0')
+from gi.repository import Gdk, GdkPixbuf, GExiv2, Gio, GLib, Pango  # isort:skip
+# fmt: on
 
-VARIETY_INFO = "-"
 
-USER_AGENT = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.143 Safari/537.36"
-
-SOURCE_NAME_TO_TYPE = {
-    'wallbase.cc': 'wallbase',
-    'wallhaven.cc': 'wallhaven',
-    'wallpapers.net': 'wn',
-    'desktoppr.co': 'desktoppr',
-    'nasa astro pic of the day': 'apod',
-    'opentopia.com': 'earth',
-    'fetched': 'fetched',
-    'recommended by variety': 'recommended',
-    'flickr': 'flickr',
-    'media rss': 'mediarss',
-}
+USER_AGENT = "Variety Wallpaper Changer " + get_version()
 
 random.seed()
-logger = logging.getLogger('variety')
+logger = logging.getLogger("variety")
+gettext.textdomain("variety")
+
+
+def _(text):
+    """Returns the translated form of text."""
+    if not text or not text.strip():
+        return text
+    return gettext.gettext(text)
 
 
 def debounce(seconds):
     """ Decorator that will postpone a functions execution until after wait seconds
         have elapsed since the last time it was invoked. """
+
     def decorator(fn):
         def debounced(*args, **kwargs):
             def call_it():
                 fn(*args, **kwargs)
+
             try:
                 debounced.t.cancel()
-            except(AttributeError):
+            except (AttributeError):
                 pass
             debounced.t = threading.Timer(seconds, call_it)
             debounced.t.start()
+
         return debounced
+
     return decorator
 
 
@@ -88,6 +93,7 @@ class throttle(object):
         def my_fun():
             pass
     """
+
     def __init__(self, seconds=0, trailing_call=False):
         """
         seconds - throttle interval in seconds
@@ -99,7 +105,7 @@ class throttle(object):
         self.timer = None
 
     def __call__(self, fn):
-        @wraps(fn)
+        @functools.wraps(fn)
         def wrapper(*args, **kwargs):
             try:
                 self.timer.cancel()
@@ -120,28 +126,29 @@ class throttle(object):
         return wrapper
 
 
-def cache(ttl_seconds=100*365*24*3600, debug=False):
+def cache(ttl_seconds=100 * 365 * 24 * 3600, debug=False):
     """
     caching decorator with TTL. Keep in mind the cache is per-process.
     TODO: There is no process for cache invalidation now. Introduce memcached and use it instead.
     :param ttl_seconds: TTL in seconds before the cache entry expires
     :param debug: use True to log cache hits (with DEBUG level)
     """
+
     def decorate(f):
         _cache = {}
 
-        @wraps(f)
+        @functools.wraps(f)
         def decorated(*args):
             cached = _cache.get(args)
-            if not cached or cached['timestamp'] < datetime.datetime.now() - datetime.timedelta(seconds=ttl_seconds):
-                cached = {
-                    'timestamp': datetime.datetime.now(),
-                    'result': f(*args)
-                }
+            if not cached or cached["timestamp"] < datetime.datetime.now() - datetime.timedelta(
+                seconds=ttl_seconds
+            ):
+                cached = {"timestamp": datetime.datetime.now(), "result": f(*args)}
                 _cache[args] = cached
             elif debug:
-                logger.debug(lambda: '@cache hit for %s' % str(args))
-            return cached['result']
+                logger.debug(lambda: "@cache hit for %s" % str(args))
+            return cached["result"]
+
         return decorated
 
     return decorate
@@ -175,7 +182,7 @@ class VarietyMetadata(GExiv2.Metadata):
             else:
                 return self.get_tag_string(key)
         else:
-            raise KeyError('%s: Unknown tag' % key)
+            raise KeyError("%s: Unknown tag" % key)
 
     def __setitem__(self, key, value):
         if key in self.MULTIPLES:
@@ -185,7 +192,8 @@ class VarietyMetadata(GExiv2.Metadata):
         else:
             self.set_tag_string(key, value)
 
-class ModuleProfiler():
+
+class ModuleProfiler:
     # How deep in other modules' code we should profile
     MAX_NONTARGET_DEPTH = 1
 
@@ -205,8 +213,12 @@ class ModuleProfiler():
         """
         modulename = cls.__module__
         if modulename not in sys.modules:
-            logger.error("ModuleProfiler: Could not add module %r (class %s) to the list of modules to trace - "
-                         "has it been imported entirely?", modulename, cls)
+            logger.error(
+                "ModuleProfiler: Could not add module %r (class %s) to the list of modules to trace - "
+                "has it been imported entirely?",
+                modulename,
+                cls,
+            )
             return
 
         module = sys.modules[modulename]
@@ -225,8 +237,9 @@ class ModuleProfiler():
         """
         self.target_paths.append(path)
 
-        logger.info("ModuleProfiler: added path %s to list of profile targets (request=%s)",
-                    path, request)
+        logger.info(
+            "ModuleProfiler: added path %s to list of profile targets (request=%s)", path, request
+        )
 
     @functools.lru_cache(maxsize=2048)
     def is_target_path(self, path):
@@ -234,7 +247,7 @@ class ModuleProfiler():
         Returns whether the given path matches one of our modules to be profiled.
         """
         for target in self.target_paths:
-            if os.path.isdir(target) and path.startswith(target+os.path.sep):
+            if os.path.isdir(target) and path.startswith(target + os.path.sep):
                 return True
             elif path == target:
                 return True
@@ -251,7 +264,9 @@ class ModuleProfiler():
         Removes the module profiler globally and from future threads.
         """
         if sys.getprofile() != self.profiler:
-            logger.warning("ModuleProfiler: The currently enabled profile function was not ours - unbinding anyways")
+            logger.warning(
+                "ModuleProfiler: The currently enabled profile function was not ours - unbinding anyways"
+            )
         threading.setprofile(None)
         sys.setprofile(None)
 
@@ -263,7 +278,9 @@ class ModuleProfiler():
         if not self.is_target_path(filename):
             if tid not in self.nontarget_depths:
                 # Pick up where the main thread left off
-                self.nontarget_depths[tid] = self.nontarget_depths.get(threading.main_thread().ident, 1)
+                self.nontarget_depths[tid] = self.nontarget_depths.get(
+                    threading.main_thread().ident, 1
+                )
             else:
                 self.nontarget_depths[tid] += 1
         else:
@@ -271,39 +288,54 @@ class ModuleProfiler():
 
         tname = threading.current_thread().name
 
-        if event == 'call':
+        if event == "call":
             if self.nontarget_depths[tid] > self.MAX_NONTARGET_DEPTH:
                 # Don't log past our max depth for packages that we're not tracking
                 return
             else:
                 # In order: function name, line number, filename
-                s = '[%s] -> Entering function: %s\t(line %s in %s)' % \
-                   (tname, frame.f_code.co_name, frame.f_lineno, filename)
+                s = "[%s] -> Entering function: %s\t(line %s in %s)" % (
+                    tname,
+                    frame.f_code.co_name,
+                    frame.f_lineno,
+                    filename,
+                )
                 if self.nontarget_depths[tid] == self.MAX_NONTARGET_DEPTH:
-                    s += " - not tracing further because MAX_NONTARGET_DEPTH=%s" % self.MAX_NONTARGET_DEPTH
+                    s += (
+                        " - not tracing further because MAX_NONTARGET_DEPTH=%s"
+                        % self.MAX_NONTARGET_DEPTH
+                    )
                 logger.debug(s)
 
-
-        elif event == 'return':
+        elif event == "return":
             if self.nontarget_depths[tid] > self.MAX_NONTARGET_DEPTH:
                 return
 
-            logger.debug('[%s] -> Leaving function:  %s\t(line %s in %s)' %
-                         (tname, frame.f_code.co_name, frame.f_lineno, filename))
+            logger.debug(
+                "[%s] -> Leaving function:  %s\t(line %s in %s)"
+                % (tname, frame.f_code.co_name, frame.f_lineno, filename)
+            )
+
+
+class InternetDisabledError(Exception):
+    pass
+
 
 class Util:
+    internet_enabled = True
+
     @staticmethod
     def sanitize_filename(filename):
         valid_chars = " ,.!-+@()_%s%s" % (string.ascii_letters, string.digits)
-        return ''.join(c if c in valid_chars else '_' for c in filename)
+        return "".join(c if c in valid_chars else "_" for c in filename)
 
     @staticmethod
     def get_local_name(url, ensure_image=True):
-        filename = url[url.rfind('/') + 1:]
-        index = filename.find('?')
+        filename = url[url.rfind("/") + 1 :]
+        index = filename.find("?")
         if index > 0:
             filename = filename[:index]
-        index = filename.find('#')
+        index = filename.find("#")
         if index > 0:
             filename = filename[:index]
 
@@ -315,12 +347,12 @@ class Util:
             filename = filename[:190] + filename[-10:]
 
         if ensure_image and not Util.is_image(filename):
-            filename += '.jpg'
+            filename += ".jpg"
 
         return filename
 
     @staticmethod
-    def split(s, seps=(',', ' ')):
+    def split(s, seps=(",", " ")):
         result = s.split()
         for sep in seps:
             result = [x.strip() for y in result for x in y.split(sep) if x.strip()]
@@ -341,14 +373,16 @@ class Util:
             return False
 
         if not check_contents:
-            return filename.lower().endswith(('.jpg', '.jpeg', '.gif', '.png', '.tiff', '.svg', '.bmp'))
+            return filename.lower().endswith(
+                (".jpg", ".jpeg", ".gif", ".png", ".tiff", ".svg", ".bmp")
+            )
         else:
             format, image_width, image_height = GdkPixbuf.Pixbuf.get_file_info(filename)
             return bool(format)
 
     @staticmethod
     def is_animated_gif(filename):
-        if not filename.lower().endswith('.gif'):
+        if not filename.lower().endswith(".gif"):
             return False
 
         gif = Image.open(filename)
@@ -360,10 +394,14 @@ class Util:
             return True
 
     @staticmethod
-    def list_files(files=(), folders=(), filter_func=(lambda f: True), max_files=10000, randomize=True):
+    def list_files(
+        files=(), folders=(), filter_func=(lambda f: True), max_files=10000, randomize=True
+    ):
         count = 0
         for filepath in files:
-            logger.debug(lambda: 'checking file %s against filter_func %s' % (filepath, filter_func))
+            logger.debug(
+                lambda: "checking file %s against filter_func %s" % (filepath, filter_func)
+            )
             if filter_func(filepath) and os.access(filepath, os.R_OK):
                 count += 1
                 yield filepath
@@ -375,17 +413,23 @@ class Util:
         for folder in folders:
             if os.path.isdir(folder):
                 try:
-                    for root, subFolders, files in os.walk(folder):
+                    for root, subFolders, files in os.walk(folder, followlinks=True):
                         if randomize:
                             random.shuffle(files)
                             random.shuffle(subFolders)
                         for filename in files:
-                            logger.debug(lambda: 'checking file %s against filter_func %s (root=%s)' % (filename, filter_func, root))
+                            logger.debug(
+                                lambda: "checking file %s against filter_func %s (root=%s)"
+                                % (filename, filter_func, root)
+                            )
                             path = os.path.join(root, filename)
                             if filter_func(path):
                                 count += 1
                                 if count > max_files:
-                                    logger.info(lambda: "More than %d files in the folders, stop listing" % max_files)
+                                    logger.info(
+                                        lambda: "More than %d files in the folders, stop listing"
+                                        % max_files
+                                    )
                                     return
                                 yield path
                 except Exception:
@@ -397,6 +441,7 @@ class Util:
             time.sleep(delay)
             print("Exiting takes too long. Calling os.kill.")
             os.kill(os.getpid(), 9)
+
         force_exit_thread = threading.Thread(target=force_exit)
         force_exit_thread.daemon = True
         force_exit_thread.start()
@@ -406,25 +451,25 @@ class Util:
         try:
             m = VarietyMetadata(filename)
             for k, v in sorted(info.items()):
-                if k == 'author':
+                if k == "author":
                     m["Xmp.variety." + k] = v
-                    if not 'Xmp.dc.creator' in m:
-                        m['Xmp.dc.creator'] = [v]
-                if k == 'headline':
-                    m['Iptc.Application2.Headline'] = [v]
-                elif k == 'description':
+                    if not "Xmp.dc.creator" in m:
+                        m["Xmp.dc.creator"] = [v]
+                if k == "headline":
+                    m["Iptc.Application2.Headline"] = [v]
+                elif k == "description":
                     if v is not None:
                         m.set_comment(v)
                     else:
                         m.clear_comment()
-                elif k == 'keywords':
+                elif k == "keywords":
                     if not isinstance(v, (list, tuple)):
                         v = [v]
-                    m['Iptc.Application2.Keywords'] = v
-                    m['Xmp.dc.subject'] = v
-                elif k == 'sfwRating':
+                    m["Iptc.Application2.Keywords"] = v
+                    m["Xmp.dc.subject"] = v
+                elif k == "sfwRating":
                     m["Xmp.variety." + k] = int(v)
-                elif k == 'extraData':
+                elif k == "extraData":
                     m["Xmp.variety." + k] = json.dumps(v, sort_keys=True)
                 else:
                     m["Xmp.variety." + k] = v
@@ -432,9 +477,12 @@ class Util:
             return True
         except Exception as ex:
             # could not write metadata inside file, use json instead
-            logger.exception(lambda: "Could not write metadata directly in file, trying json metadata: " + filename)
+            logger.exception(
+                lambda: "Could not write metadata directly in file, trying json metadata: "
+                + filename
+            )
             try:
-                with open(filename + '.metadata.json', 'w', encoding='utf8') as f:
+                with open(filename + ".metadata.json", "w", encoding="utf8") as f:
                     f.write(json.dumps(info, indent=4, ensure_ascii=False, sort_keys=True))
                     return True
             except Exception as e:
@@ -447,40 +495,49 @@ class Util:
             m = VarietyMetadata(filename)
 
             info = {}
-            for k in ["sourceName", "sourceLocation", "sourceURL", "sourceType", "imageURL", "author", "authorURL"]:
+            for k in [
+                "sourceName",
+                "sourceLocation",
+                "sourceURL",
+                "sourceType",
+                "imageURL",
+                "author",
+                "authorURL",
+                "noOriginPage",
+            ]:
                 if "Xmp.variety." + k in m:
                     info[k] = m["Xmp.variety." + k]
 
             try:
-                info['sfwRating'] = int(m['Xmp.variety.sfwRating'])
+                info["sfwRating"] = int(m["Xmp.variety.sfwRating"])
             except:
                 pass
 
             try:
-                info['author'] = m['Xmp.dc.creator'][0]
+                info["author"] = m["Xmp.dc.creator"][0]
             except:
                 pass
 
             try:
-                info['headline'] = m['Iptc.Application2.Headline'][0]
+                info["headline"] = m["Iptc.Application2.Headline"][0]
             except:
                 pass
 
             try:
-                info['description'] = m.get_comment()
+                info["description"] = m.get_comment()
             except:
                 pass
 
             try:
-                info['extraData'] = json.loads(m['Xmp.variety.extraData'])
+                info["extraData"] = json.loads(m["Xmp.variety.extraData"])
             except:
                 pass
 
             try:
-                info['keywords'] = m['Iptc.Application2.Keywords']
+                info["keywords"] = m["Iptc.Application2.Keywords"]
             except:
                 try:
-                    info['keywords'] = m['Xmp.dc.subject']
+                    info["keywords"] = m["Xmp.dc.subject"]
                 except:
                     pass
 
@@ -489,7 +546,7 @@ class Util:
         except Exception as e:
             # could not read metadata inside file, try reading json metadata instead
             try:
-                with open(filename + '.metadata.json', encoding='utf8') as f:
+                with open(filename + ".metadata.json", encoding="utf8") as f:
                     return json.loads(f.read())
 
             except Exception:
@@ -534,13 +591,13 @@ class Util:
     def get_size(image):
         format, image_width, image_height = GdkPixbuf.Pixbuf.get_file_info(image)
         if not format:
-            raise Exception('Not an image or unsupported image format')
+            raise Exception("Not an image or unsupported image format")
         else:
             return image_width, image_height
 
     @staticmethod
     def find_unique_name(filename):
-        index = filename.rfind('.')
+        index = filename.rfind(".")
         if index < 0:
             index = len(filename)
         before_extension = filename[:index]
@@ -548,31 +605,34 @@ class Util:
         i = 1
         f = filename
         while os.path.exists(f):
-            f = before_extension + '_' + str(i) + extension
+            f = before_extension + "_" + str(i) + extension
             i += 1
         return f
 
     @staticmethod
-    def request(url, data=None, stream=False, method=None, timeout=5):
-        if url.startswith('//'):
-            url = 'http:' + url
-        headers = {
-            'User-Agent': USER_AGENT,
-            'Cache-Control': 'max-age=0'
-        }
-        method = method if method else 'POST' if data else 'GET'
+    def request(url, data=None, stream=False, method=None, timeout=5, headers=None):
+        if not Util.internet_enabled:
+            raise InternetDisabledError("Internet access in Variety is currently disabled")
+
+        if url.startswith("//"):
+            url = "http:" + url
+        headers = headers or {}
+        headers = {"User-Agent": USER_AGENT, "Cache-Control": "max-age=0", **headers}
+        method = method if method else "POST" if data else "GET"
         try:
-            r = requests.request(method=method,
-                                 url=url,
-                                 data=data,
-                                 headers=headers,
-                                 stream=stream,
-                                 allow_redirects=True,
-                                 timeout=timeout)
+            r = requests.request(
+                method=method,
+                url=url,
+                data=data,
+                headers=headers,
+                stream=stream,
+                allow_redirects=True,
+                timeout=timeout,
+            )
             r.raise_for_status()
             return r
         except requests.exceptions.SSLError:
-            logger.exception('SSL Error for url %s:' % url)
+            logger.exception("SSL Error for url %s:" % url)
             raise
 
     @staticmethod
@@ -581,24 +641,29 @@ class Util:
             f.write(chunk)
 
     @staticmethod
-    def fetch(url, data=None):
-        return Util.request(url, data).text
+    def fetch(url, data=None, **request_kwargs):
+        return Util.request(url, data, **request_kwargs).text
 
     @staticmethod
-    def fetch_bytes(url, data=None):
-        return Util.request(url, data).content
+    def fetch_bytes(url, data=None, **request_kwargs):
+        return Util.request(url, data, **request_kwargs).content
 
     @staticmethod
-    def fetch_json(url, data=None):
-        return Util.request(url, data).json()
+    def fetch_json(url, data=None, **request_kwargs):
+        return Util.request(url, data, **request_kwargs).json()
 
     @staticmethod
-    def html_soup(url, data=None):
-        return bs4.BeautifulSoup(Util.fetch(url, data), "lxml")
+    def html_soup(url, data=None, **request_kwargs):
+        return bs4.BeautifulSoup(Util.fetch(url, data, **request_kwargs), "lxml")
 
     @staticmethod
-    def xml_soup(url, data=None):
-        return bs4.BeautifulSoup(Util.fetch(url, data), "xml")
+    def xml_soup(url, data=None, **request_kwargs):
+        return bs4.BeautifulSoup(Util.fetch(url, data, **request_kwargs), "xml")
+
+    @staticmethod
+    def unxor(text, key):
+        ciphertext = base64.decodebytes(text)
+        return "".join(chr(x ^ ord(y)) for (x, y) in zip(ciphertext, cycle(key)))
 
     @staticmethod
     def folderpath(folder):
@@ -616,47 +681,61 @@ class Util:
         screen_w, screen_h = screen_size
         screen_ratio = float(screen_w) / screen_h
         hoffset = voffset = 0
-        if screen_ratio > float(iw) / ih: #image is "taller" than the screen ratio - need to offset vertically
+        if (
+            screen_ratio > float(iw) / ih
+        ):  # image is "taller" than the screen ratio - need to offset vertically
             scaledw = float(screen_w)
             scaledh = ih * scaledw / iw
             voffset = int((scaledh - float(scaledw) / screen_ratio) / 2)
-        else: #image is "wider" than the screen ratio - need to offset horizontally
+        else:  # image is "wider" than the screen ratio - need to offset horizontally
             scaledh = float(screen_h)
             scaledw = iw * scaledh / ih
             hoffset = int((scaledw - float(scaledh) * screen_ratio) / 2)
 
-        logger.info(lambda: "Trimmed offsets debug info: w:%d, h:%d, ratio:%f, iw:%d, ih:%d, scw:%d, sch:%d, ho:%d, vo:%d" % (
-            screen_w, screen_h, screen_ratio, iw, ih, scaledw, scaledh, hoffset, voffset))
+        logger.info(
+            lambda: "Trimmed offsets debug info: w:%d, h:%d, ratio:%f, iw:%d, ih:%d, scw:%d, sch:%d, ho:%d, vo:%d"
+            % (screen_w, screen_h, screen_ratio, iw, ih, scaledw, scaledh, hoffset, voffset)
+        )
         return hoffset, voffset
 
     @staticmethod
     def get_scaled_size(image):
         """Computes the size to which the image is scaled to fit the screen: original_size * scale_ratio = scaled_size"""
         iw, ih = Util.get_size(image)
-        screen_w, screen_h = Gdk.Screen.get_default().get_width(), Gdk.Screen.get_default().get_height()
+        screen_w, screen_h = (
+            Gdk.Screen.get_default().get_width(),
+            Gdk.Screen.get_default().get_height(),
+        )
         screen_ratio = float(screen_w) / screen_h
-        if screen_ratio > float(iw) / ih: #image is "taller" than the screen ratio - need to offset vertically
+        if (
+            screen_ratio > float(iw) / ih
+        ):  # image is "taller" than the screen ratio - need to offset vertically
             return screen_w, int(round(ih * float(screen_w) / iw))
-        else: #image is "wider" than the screen ratio - need to offset horizontally
+        else:  # image is "wider" than the screen ratio - need to offset horizontally
             return int(round(iw * float(screen_h) / ih)), screen_h
 
     @staticmethod
     def get_scale_to_screen_ratio(image):
         """Computes the ratio by which the image is scaled to fit the screen: original_size * scale_ratio = scaled_size"""
         iw, ih = Util.get_size(image)
-        screen_w, screen_h = Gdk.Screen.get_default().get_width(), Gdk.Screen.get_default().get_height()
+        screen_w, screen_h = (
+            Gdk.Screen.get_default().get_width(),
+            Gdk.Screen.get_default().get_height(),
+        )
         screen_ratio = float(screen_w) / screen_h
-        if screen_ratio > float(iw) / ih: #image is "taller" than the screen ratio - need to offset vertically
+        if (
+            screen_ratio > float(iw) / ih
+        ):  # image is "taller" than the screen ratio - need to offset vertically
             return int(float(screen_w) / iw)
-        else: #image is "wider" than the screen ratio - need to offset horizontally
+        else:  # image is "wider" than the screen ratio - need to offset horizontally
             return int(float(screen_h) / ih)
 
     @staticmethod
     def gtk_to_fcmatch_font(gtk_font_name):
         fd = Pango.FontDescription(gtk_font_name)
         family = fd.get_family()
-        size = gtk_font_name[gtk_font_name.rindex(' '):].strip()
-        rest = gtk_font_name.replace(family, '').strip().replace(' ', ':')
+        size = gtk_font_name[gtk_font_name.rindex(" ") :].strip()
+        rest = gtk_font_name.replace(family, "").strip().replace(" ", ":")
         return family + ":" + rest, size
 
     @staticmethod
@@ -669,12 +748,13 @@ class Util:
 
     @staticmethod
     def collapseuser(path):
-        home = os.path.expanduser('~') + '/'
-        return re.sub('^' + home, '~/', path)
+        home = os.path.expanduser("~") + "/"
+        return re.sub("^" + home, "~/", path)
 
     @staticmethod
     def compare_versions(v1, v2):
         from pkg_resources import parse_version
+
         pv1 = parse_version(v1)
         pv2 = parse_version(v2)
 
@@ -686,20 +766,20 @@ class Util:
     @staticmethod
     def md5(s):
         if isinstance(s, str):
-            s = s.encode('utf-8')
+            s = s.encode("utf-8")
         return hashlib.md5(s).hexdigest()
 
     @staticmethod
     def md5file(file):
-        with open(file, mode='rb') as f:
+        with open(file, mode="rb") as f:
             return Util.md5(f.read())
 
     @staticmethod
     def random_hash():
         try:
-            return codecs.encode(os.urandom(16), 'hex_codec').decode('utf8')
+            return codecs.encode(os.urandom(16), "hex_codec").decode("utf8")
         except Exception:
-            return ''.join(random.choice(string.hexdigits) for _ in range(32)).lower()
+            return "".join(random.choice(string.hexdigits) for _ in range(32)).lower()
 
     @staticmethod
     def get_file_icon_name(path):
@@ -713,7 +793,7 @@ class Util:
 
     @staticmethod
     def is_home_encrypted():
-        return os.path.isdir(os.path.expanduser("~").replace('/home/', '/home/.ecryptfs/'))
+        return os.path.isdir(os.path.expanduser("~").replace("/home/", "/home/.ecryptfs/"))
 
     @staticmethod
     def get_xdg_pictures_folder():
@@ -723,8 +803,10 @@ class Util:
                 raise Exception("Could not get path to Pictures folder. Defaulting to ~/Pictures.")
             return pics_folder
         except:
-            logger.exception(lambda: "Could not get path to Pictures folder. Defaulting to ~/Pictures.")
-            return os.path.expanduser('~/Pictures')
+            logger.exception(
+                lambda: "Could not get path to Pictures folder. Defaulting to ~/Pictures."
+            )
+            return os.path.expanduser("~/Pictures")
 
     @staticmethod
     def superuser_exec(*command_args):
@@ -742,13 +824,13 @@ class Util:
     @staticmethod
     def get_thumbnail_data(image, width, height):
         pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(image, width, height)
-        return pixbuf.save_to_bufferv('jpeg', [], [])[1]
+        return pixbuf.save_to_bufferv("jpeg", [], [])[1]
 
     @staticmethod
     def is_alive_and_image(url):
         try:
-            r = Util.request(url, method='head')
-            return r.headers.get('content-type', '').startswith('image/')
+            r = Util.request(url, method="head")
+            return r.headers.get("content-type", "").startswith("image/")
         except Exception:
             return False
 
@@ -759,73 +841,35 @@ class Util:
 
         try:
             host = urllib.parse.urlparse(url).netloc
-            if host.startswith('interfacelift.com'):
+            if host.startswith("interfacelift.com"):
                 return False
 
-            if 'wallbase.cc' in host or 'ns223506.ovh.net' in host:
+            if "wallbase.cc" in host or "ns223506.ovh.net" in host:
                 return True
         except:
             return True
 
         try:
-            r = Util.request(url, method='head')
-            return not r.headers.get('content-type', '').startswith('image/')
+            r = Util.request(url, method="head")
+            return not r.headers.get("content-type", "").startswith("image/")
         except requests.exceptions.RequestException:
             return True
         except:
             return False
 
-    @staticmethod
-    def guess_image_url(meta):
-        if 'imageURL' in meta:
-            return meta['imageURL']
-
-        try:
-            origin_url = meta['sourceURL']
-
-            if "flickr.com" in origin_url:
-                from variety.FlickrDownloader import FlickrDownloader
-                return FlickrDownloader.get_image_url(origin_url)
-
-            elif Util.is_image(origin_url) and Util.is_alive_and_image(origin_url):
-                return origin_url
-
-            return None
-        except:
-            return None
-
-    @staticmethod
-    def guess_source_type(meta):
-        try:
-            if 'sourceType' in meta:
-                return meta['sourceType']
-            elif 'sourceName' in meta:
-                source_name = meta['sourceName'].lower()
-                if source_name in SOURCE_NAME_TO_TYPE:
-                    return SOURCE_NAME_TO_TYPE[source_name]
-                else:
-                    source_location = meta.get('sourceLocation', '').lower()
-                    if source_location.startswith(('http://' + source_name, 'https://' + source_name)) \
-                            or 'backend.deviantart.com' in source_location \
-                            or 'rss' in source_location \
-                            or '/feed' in source_location:
-                        return 'mediarss'
-            return None
-        except:
-            return None
-
     # makes the Gtk thread execute the given callback.
     @staticmethod
     def add_mainloop_task(callback, *args):
         def cb(args):
             args[0](*args[1:])
             return False
-        args= [callback]+list(args)
+
+        args = [callback] + list(args)
         Gdk.threads_add_idle(GLib.PRIORITY_DEFAULT, cb, args)
 
     @staticmethod
     def is_unity():
-        return os.getenv('XDG_CURRENT_DESKTOP', '').lower() == 'unity'
+        return os.getenv("XDG_CURRENT_DESKTOP", "").lower() == "unity"
 
     @staticmethod
     def start_daemon(target):
@@ -836,10 +880,80 @@ class Util:
 
     @staticmethod
     def check_variety_slideshow_present():
-        try:
-            subprocess.check_call(['which', 'variety-slideshow'],
-                                  stderr=subprocess.PIPE, stdout=subprocess.PIPE)
-            return True
-        except:
-            return False
+        return bool(shutil.which("variety-slideshow"))
 
+    @staticmethod
+    def convert_to_filename(url):
+        url = re.sub(r"http://", "", url)
+        url = re.sub(r"https://", "", url)
+        valid_chars = "_%s%s" % (string.ascii_letters, string.digits)
+        return "".join(c if c in valid_chars else "_" for c in url)
+
+    @staticmethod
+    def safe_unlink(filepath):
+        try:
+            os.unlink(filepath)
+        except Exception:
+            logger.exception(lambda: "Could not delete {}, ignoring".format(filepath))
+
+    @staticmethod
+    def copy_with_replace(from_path, to_path, search_replace_map):
+        with open(from_path, "r") as file:
+            data = file.read()
+        for search, replace in search_replace_map.items():
+            data = data.replace(search, replace)
+        with open(to_path + ".partial", "w") as file:
+            file.write(data)
+            file.flush()
+        os.rename(to_path + ".partial", to_path)
+
+    @staticmethod
+    def get_exec_path():
+        return os.path.abspath(sys.argv[0])
+
+    @staticmethod
+    def get_folder_size(start_path):
+        total_size = 0
+        for dirpath, dirnames, filenames in os.walk(start_path):
+            for f in filenames:
+                fp = os.path.join(dirpath, f)
+                if not os.path.islink(fp):
+                    total_size += os.path.getsize(fp)
+        return total_size
+
+    @staticmethod
+    def get_screen_width():
+        return Gdk.Screen.get_default().get_width()
+
+
+def on_gtk(f):
+    @functools.wraps(f)
+    def wrapped(*args):
+        Util.add_mainloop_task(f, *args)
+
+    return wrapped
+
+
+def safe_print(text, ascii_text=None, file=sys.stdout):
+    """
+    Python's print throws UnicodeEncodeError if the terminal encoding is borked. This version tries print, then logging, then printing the ascii text when one is present.
+    If does not throw exceptions even if it fails.
+    :param text: Text to print, str or unicode, possibly with non-ascii symbols in it
+    :param ascii_text: optional. Original untranslated ascii version of the text when present.
+    """
+    try:
+        print(text, file=file)
+    except:  # UnicodeEncodeError can happen here if the terminal is strangely configured, but we are playing safe and catching everything
+        try:
+            logging.getLogger("variety").error(
+                "Error printing non-ascii text, terminal encoding is %s" % sys.stdout.encoding
+            )
+            if ascii_text:
+                try:
+                    print(ascii_text)
+                    return
+                except:
+                    pass
+            logging.getLogger("variety").warning(text)
+        except:
+            pass
